@@ -1,5 +1,5 @@
 .onAttach <- function(libname, pkgname) {
-  packageStartupMessage("For news and documentation about the R for Photobiology packages, please, visit http://www.r4photobiology.info/")
+  packageStartupMessage("For news on the R for Photobiology packages, please, visit https://www.r4photobiology.info/")
 }
 
 .onLoad <- function(libname, pkgname) {
