@@ -235,6 +235,15 @@ str(msaply(two.mspct, max))
 ## ---- apply-8------------------------------------------------------------
 str(msaply(two.mspct, range))
 
+## ------------------------------------------------------------------------
+s_mean(two.mspct)
+
+## ------------------------------------------------------------------------
+s_se(two.mspct)
+
+## ------------------------------------------------------------------------
+s_mean_se(two.mspct)
+
 ## ---- convolve-1---------------------------------------------------------
 convolve_each(two.mspct, sun.spct)
 
@@ -448,6 +457,12 @@ clean(sun.spct - 0.01, range = c(280.5, 282))
 ## ---- manip-12-----------------------------------------------------------
 clean(polyester.spct - 0.053)
 
+## ------------------------------------------------------------------------
+smooth_spct(sun.spct)
+
+## ------------------------------------------------------------------------
+smooth_spct(polyester.spct, method = "supsmu", strength = 2)
+
 ## ---- manip-13-----------------------------------------------------------
 interpolate_wl(sun.spct, seq(400, 500, by = 0.1))
 
