@@ -148,3 +148,18 @@ relative_AM(33)
 ## ------------------------------------------------------------------------
 relative_AM(c(80, 60, 40, 20))
 
+## ------------------------------------------------------------------------
+water_vp_sat(c(0, 10, 20, 30))
+
+## ------------------------------------------------------------------------
+water_vp2mvc(water_vp_sat(20), 20)
+
+## ------------------------------------------------------------------------
+water_RH2vp(50, 20)
+
+## ------------------------------------------------------------------------
+water_vp2RH(500, 40)
+
+## ------------------------------------------------------------------------
+water_dp(3000)
+
